@@ -4,50 +4,53 @@
 /* macros for various libc functions you can change for embedded targets */
 #ifndef XMALLOC
    #ifdef malloc
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XMALLOC  malloc
 #endif
 #ifndef XREALLOC
    #ifdef realloc
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XREALLOC realloc
 #endif
 #ifndef XCALLOC
    #ifdef calloc
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XCALLOC  calloc
 #endif
 #ifndef XFREE
    #ifdef free
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XFREE    free
 #endif
 
 #ifndef XMEMSET
    #ifdef memset
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XMEMSET  memset
 #endif
 #ifndef XMEMCPY
    #ifdef memcpy
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XMEMCPY  memcpy
 #endif
 #ifndef XMEMCMP
    #ifdef memcmp
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XMEMCMP  memcmp
 #endif
+#ifndef XMEM_NEQ
+#define XMEM_NEQ  mem_neq
+#endif
 #ifndef XSTRCMP
    #ifdef strcmp
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XSTRCMP strcmp
 #endif
@@ -61,11 +64,25 @@
 
 #ifndef XQSORT
    #ifdef qsort
-   /* #define LTC_NO_PROTOTYPES */
+   #define LTC_NO_PROTOTYPES
    #endif
 #define XQSORT qsort
 #endif
 
+/* shortcut to disable automatic inclusion */
+#if defined LTC_NOTHING && !defined LTC_EASY
+  #define LTC_NO_MATH
+  #define LTC_NO_CIPHERS
+  #define LTC_NO_MODES
+  #define LTC_NO_HASHES
+  #define LTC_NO_MACS
+  #define LTC_NO_PRNGS
+  #define LTC_NO_PK
+  #define LTC_NO_PKCS
+  #define LTC_NO_MISC
+  #define LTC_NO_FILE
+#endif /* LTC_NOTHING */
+
 /* Easy button? */
 #ifdef LTC_EASY
    #define LTC_NO_CIPHERS
@@ -85,6 +102,7 @@
    #define LTC_SHA384
    #define LTC_SHA256
    #define LTC_SHA224
+   #define LTC_HASH_HELPERS
 
    #define LTC_NO_MACS
    #define LTC_HMAC
@@ -95,20 +113,27 @@
    #define LTC_SPRNG
    #define LTC_YARROW
    #define LTC_DEVRANDOM
-   #define TRY_URANDOM_FIRST
+   #define LTC_TRY_URANDOM_FIRST
+   #define LTC_RNG_GET_BYTES
+   #define LTC_RNG_MAKE_PRNG
 
    #define LTC_NO_PK
    #define LTC_MRSA
    #define LTC_MECC
-#endif
 
-/* Use small code where possible */
-/* #define LTC_SMALL_CODE */
+   #define LTC_NO_MISC
+   #define LTC_BASE64
+#endif
 
 /* Enable self-test test vector checking */
 #ifndef LTC_NO_TEST
    #define LTC_TEST
 #endif
+/* Enable extended self-tests */
+/* #define LTC_TEST_EXT */
+
+/* Use small code where possible */
+/* #define LTC_SMALL_CODE */
 
 /* clean the stack of functions which put private information on stack */
 /* #define LTC_CLEAN_STACK */
@@ -125,6 +150,20 @@
 /* disable BSWAP on x86 */
 /* #define LTC_NO_BSWAP */
 
+/* ---> math provider? <--- */
+#ifndef LTC_NO_MATH
+
+/* LibTomMath */
+/* #define LTM_DESC */
+
+/* TomsFastMath */
+/* #define TFM_DESC */
+
+#endif /* LTC_NO_MATH */
+
+/* GNU Multiple Precision Arithmetic Library */
+/* #define GMP_DESC */
+
 /* ---> Symmetric Block Ciphers <--- */
 #ifndef LTC_NO_CIPHERS
 
@@ -145,7 +184,7 @@
    #define LTC_TWOFISH_SMALL
 #endif
 /* #define LTC_TWOFISH_SMALL */
-/* LTC_DES includes EDE triple-LTC_DES */
+/* LTC_DES includes EDE triple-DES */
 #define LTC_DES
 #define LTC_CAST5
 #define LTC_NOEKEON
@@ -180,7 +219,7 @@
    /* like GCM mode this will enable 16 8x128 tables [64KB] that make
     * seeking very fast.
     */
-   #define LRW_TABLES
+   #define LTC_LRW_TABLES
 #endif
 
 /* XTS mode */
@@ -194,6 +233,8 @@
 #define LTC_CHC_HASH
 #define LTC_WHIRLPOOL
 #define LTC_SHA512
+#define LTC_SHA512_256
+#define LTC_SHA512_224
 #define LTC_SHA384
 #define LTC_SHA256
 #define LTC_SHA224
@@ -207,6 +248,8 @@
 #define LTC_RIPEMD256
 #define LTC_RIPEMD320
 
+#define LTC_HASH_HELPERS
+
 #endif /* LTC_NO_HASHES */
 
 /* ---> MAC functions <--- */
@@ -219,16 +262,9 @@
 #define LTC_F9_MODE
 #define LTC_PELICAN
 
-#if defined(LTC_PELICAN) && !defined(LTC_RIJNDAEL)
-   #error Pelican-MAC requires LTC_RIJNDAEL
-#endif
-
 /* ---> Encrypt + Authenticate Modes <--- */
 
 #define LTC_EAX_MODE
-#if defined(LTC_EAX_MODE) && !(defined(LTC_CTR_MODE) && defined(LTC_OMAC))
-   #error LTC_EAX_MODE requires CTR and LTC_OMAC mode
-#endif
 
 #define LTC_OCB_MODE
 #define LTC_OCB3_MODE
@@ -247,9 +283,6 @@
 
 #endif /* LTC_NO_MACS */
 
-/* Various tidbits of modern neatoness */
-#define LTC_BASE64
-
 /* --> Pseudo Random Number Generators <--- */
 #ifndef LTC_NO_PRNGS
 
@@ -258,13 +291,9 @@
 /* which descriptor of AES to use?  */
 /* 0 = rijndael_enc 1 = aes_enc, 2 = rijndael [full], 3 = aes [full] */
 #ifdef ENCRYPT_ONLY
-  #define LTC_YARROW_AES 3
+  #define LTC_YARROW_AES 0
 #else
-  #define LTC_YARROW_AES 3
-#endif
-
-#if defined(LTC_YARROW) && !defined(LTC_CTR_MODE)
-   #error LTC_YARROW requires LTC_CTR_MODE chaining mode to be defined!
+  #define LTC_YARROW_AES 2
 #endif
 
 /* a PRNG that simply reads from an available system source */
@@ -286,52 +315,47 @@
 /* the *nix style /dev/random device */
 #define LTC_DEVRANDOM
 /* try /dev/urandom before trying /dev/random */
-#define TRY_URANDOM_FIRST
+#define LTC_TRY_URANDOM_FIRST
+/* rng_get_bytes() */
+#define LTC_RNG_GET_BYTES
+/* rng_make_prng() */
+#define LTC_RNG_MAKE_PRNG
 
 #endif /* LTC_NO_PRNGS */
 
-/* ---> math provider? <--- */
-#ifndef LTC_NO_MATH
-
-/* LibTomMath */
-/* #define LTM_DESC */
-
-/* TomsFastMath */
-/* #define TFM_DESC */
-
-#endif /* LTC_NO_MATH */
-
 /* ---> Public Key Crypto <--- */
 #ifndef LTC_NO_PK
 
 /* Include RSA support */
 #define LTC_MRSA
 
-/* Enable RSA blinding when doing private key operations? */
-/* #define LTC_RSA_BLINDING */
+#ifndef LTC_NO_RSA_BLINDING
+/* Enable RSA blinding when doing private key operations by default */
+#define LTC_RSA_BLINDING
+#endif  /* LTC_NO_RSA_BLINDING */
 
 /* Include Diffie-Hellman support */
 #ifndef GMP_DESC
 /* is_prime fails for GMP */
-#define MDH
+#define LTC_MDH
 /* Supported Key Sizes */
-#define DH768
-#define DH1024
-#define DH1280
-#define DH1536
-#define DH1792
-#define DH2048
+#define LTC_DH768
+#define LTC_DH1024
+#define LTC_DH1280
+#define LTC_DH1536
+#define LTC_DH1792
+#define LTC_DH2048
 
 #ifndef TFM_DESC
 /* tfm has a problem in fp_isprime for larger key sizes */
-#define DH2560
-#define DH3072
-#define DH4096
+#define LTC_DH2560
+#define LTC_DH3072
+#define LTC_DH4096
 #endif
 #endif
 
 /* Include Katja (a Rabin variant like RSA) */
-/* #define MKAT */
+/* #define LTC_MKAT */
 
 /* Digital Signature Algorithm */
 #define LTC_MDSA
@@ -342,19 +366,45 @@
 /* use Shamir's trick for point mul (speeds up signature verification) */
 #define LTC_ECC_SHAMIR
 
-#if defined(TFM_LTC_DESC) && defined(LTC_MECC)
+#if defined(TFM_DESC) && defined(LTC_MECC)
    #define LTC_MECC_ACCEL
 #endif
 
 /* do we want fixed point ECC */
 /* #define LTC_MECC_FP */
 
-/* Timing Resistant? */
+#ifndef LTC_NO_ECC_TIMING_RESISTANT
+/* Enable ECC timing resistant version by default */
 #define LTC_ECC_TIMING_RESISTANT
+#endif
 
 #endif /* LTC_NO_PK */
 
-/* LTC_PKCS #1 (RSA) and #5 (Password Handling) stuff */
+/* define these PK sizes out of LTC_NO_PK
+ * to have them always defined
+ */
+#if defined(LTC_MRSA)
+/* Min and Max RSA key sizes (in bits) */
+#ifndef MIN_RSA_SIZE
+#define MIN_RSA_SIZE 1024
+#endif
+#ifndef MAX_RSA_SIZE
+#define MAX_RSA_SIZE 4096
+#endif
+#endif
+
+/* in cases where you want ASN.1/DER functionality, but no
+ * RSA, you can define this externally if 1024 is not enough
+ */
+#if defined(LTC_MRSA)
+#define LTC_DER_MAX_PUBKEY_SIZE MAX_RSA_SIZE
+#elif !defined(LTC_DER_MAX_PUBKEY_SIZE)
+/* this includes DSA */
+#define LTC_DER_MAX_PUBKEY_SIZE 1024
+#endif
+
+
+/* PKCS #1 (RSA) and #5 (Password Handling) stuff */
 #ifndef LTC_NO_PKCS
 
 #define LTC_PKCS_1
@@ -365,32 +415,70 @@
 
 #endif /* LTC_NO_PKCS */
 
-/* LTC_HKDF Key Derivation/Expansion stuff */
+/* misc stuff */
+#ifndef LTC_NO_MISC
+
+/* Various tidbits of modern neatoness */
+#define LTC_BASE64
+/* ... and it's URL safe version */
+#define LTC_BASE64_URL
+
+/* Keep LTC_NO_HKDF for compatibility reasons
+ * superseeded by LTC_NO_MISC*/
 #ifndef LTC_NO_HKDF
-
+/* HKDF Key Derivation/Expansion stuff */
 #define LTC_HKDF
-
 #endif /* LTC_NO_HKDF */
 
+#endif /* LTC_NO_MISC */
+
 /* cleanup */
 
 #ifdef LTC_MECC
 /* Supported ECC Key Sizes */
 #ifndef LTC_NO_CURVES
-   #define ECC112
-   #define ECC128
-   #define ECC160
-   #define ECC192
-   #define ECC224
-   #define ECC256
-   #define ECC384
-   #define ECC521
+   #define LTC_ECC_SECP112R1
+   #define LTC_ECC_SECP112R2
+   #define LTC_ECC_SECP128R1
+   #define LTC_ECC_SECP128R2
+   #define LTC_ECC_SECP160R1
+   #define LTC_ECC_SECP160R2
+   #define LTC_ECC_SECP160K1
+   #define LTC_ECC_BRAINPOOLP160R1
+   #define LTC_ECC_SECP192R1
+   #define LTC_ECC_PRIME192V2
+   #define LTC_ECC_PRIME192V3
+   #define LTC_ECC_SECP192K1
+   #define LTC_ECC_BRAINPOOLP192R1
+   #define LTC_ECC_SECP224R1
+   #define LTC_ECC_SECP224K1
+   #define LTC_ECC_BRAINPOOLP224R1
+   #define LTC_ECC_PRIME239V1
+   #define LTC_ECC_PRIME239V2
+   #define LTC_ECC_PRIME239V3
+   #define LTC_ECC_SECP256R1
+   #define LTC_ECC_SECP256K1
+   #define LTC_ECC_BRAINPOOLP256R1
+   #define LTC_ECC_BRAINPOOLP320R1
+   #define LTC_ECC_SECP384R1
+   #define LTC_ECC_BRAINPOOLP384R1
+   #define LTC_ECC_BRAINPOOLP512R1
+   #define LTC_ECC_SECP521R1
+   /* OLD deprecated (but still working) defines */
+   #define LTC_ECC112
+   #define LTC_ECC128
+   #define LTC_ECC160
+   #define LTC_ECC192
+   #define LTC_ECC224
+   #define LTC_ECC256
+   #define LTC_ECC384
+   #define LTC_ECC521
 #endif
 #endif
 
-#if defined(LTC_MECC) || defined(LTC_MRSA) || defined(LTC_MDSA) || defined(MKATJA)
+#if defined(LTC_MECC) || defined(LTC_MRSA) || defined(LTC_MDSA) || defined(LTC_MKAT)
    /* Include the MPI functionality?  (required by the PK algorithms) */
-   #define MPI
+   #define LTC_MPI
 #endif
 
 #ifdef LTC_MRSA
@@ -402,11 +490,23 @@
     #undef LTC_RSA_BLINDING
 #endif
 
-#if defined(LTC_DER) && !defined(MPI)
+#if defined(LTC_PELICAN) && !defined(LTC_RIJNDAEL)
+   #error Pelican-MAC requires LTC_RIJNDAEL
+#endif
+
+#if defined(LTC_EAX_MODE) && !(defined(LTC_CTR_MODE) && defined(LTC_OMAC))
+   #error LTC_EAX_MODE requires CTR and LTC_OMAC mode
+#endif
+
+#if defined(LTC_YARROW) && !defined(LTC_CTR_MODE)
+   #error LTC_YARROW requires LTC_CTR_MODE chaining mode to be defined!
+#endif
+
+#if defined(LTC_DER) && !defined(LTC_MPI)
    #error ASN.1 DER requires MPI functionality
 #endif
 
-#if (defined(LTC_MDSA) || defined(LTC_MRSA) || defined(LTC_MECC) || defined(MKATJA)) && !defined(LTC_DER)
+#if (defined(LTC_MDSA) || defined(LTC_MRSA) || defined(LTC_MECC) || defined(LTC_MKAT)) && !defined(LTC_DER)
    #error PK requires ASN.1 DER functionality, make sure LTC_DER is enabled
 #endif
 
