@@ -79,6 +79,8 @@
 #define BN_MP_PRIME_IS_DIVISIBLE_C
 #define BN_MP_PRIME_IS_PRIME_C
 #define BN_MP_PRIME_MILLER_RABIN_C
+#define BN_MP_PRIME_MILLER_RABIN_RANDOM_C
+#define BN_MP_PRIME_LUCAS_C
 #define BN_MP_PRIME_NEXT_PRIME_C
 #define BN_MP_PRIME_RABIN_MILLER_TRIALS_C
 #define BN_MP_PRIME_RANDOM_EX_C
